define([
    // dojo
    "dojo/_base/declare",
    "dojo/dom-construct",
    "dojo/string",
    "dojo/when",

    // epi.shell
    "epi/dependency",
    "epi/shell/TypeDescriptorManager",

    // epi.cms
    "epi-cms/core/ContentReference",
    "epi-cms/contentediting/ContentActionSupport",
    "epi-cms/widget/_SelectorBase",

    // resources

    "epi/i18n!epi/cms/nls/episerver.cms.widget.contentselector"
], function (
    // dojo
    declare,
    domConstruct,
    stringUtil,
    when,

    // epi.shell
    dependency,
    TypeDescriptorManager,

    // epi.cms
    ContentReference,
    ContentActionSupport,
    _SelectorBase,

    // resources
    localization
) {

    return declare([_SelectorBase], {
        // summary:
        //    Represents the widget to select content reference.
        // tags:
        //      internal xproduct

        roots: null,

        value: null,

        // destroyOnHide: [protected] Boolean
        //      Flag which indicates whether the content selector dialog will be destroyed after being hidden.
        destroyOnHide: true,

        // required: Boolean
        //		User is required to enter data into this field.
        required: false,

        // missingMessage: [public] String
        //    Message which is displayed when required is true and value is empty.
        missingMessage: localization.requiredmessage,

        // tooltipPosition: String[]
        //		See description of `dijit/Tooltip.defaultPosition` for details on this parameter.
        tooltipPosition: [],

        // contentLink: [public] String
        //      Content reference of the content being edited.
        contentLink: null,

        // canSelectOwnerContent: [public] Boolean
        //      Indicates whether the owner content that is being edited can be selected
        canSelectOwnerContent: false,

        // accessLevel: [public] Enum
        //      The access right level for the content node can be selected
        accessLevel: ContentActionSupport.accessLevel.Read,

        // showAllLanguages: Boolean
        //      Flags to indicate that the content tree should show all content in multiple languages or not.
        showAllLanguages: true,

        // showSearchBox: Boolean
        //      If true - a search field is displayed that allows user to find content
        showSearchBox: true,

        // searchArea: String
        //      Search area for the search box.
        searchArea: null,

        // contentClass: [public] String
        //      The content class to be set on the epi-cms/widget/ContentSelectorDialog.
        contentClass: null,

        // dialogClass: [public] String
        //      The dialog class to be set on the epi-cms/widget/ContentSelectorDialog.
        dialogClass: "epi-dialog-portrait",

        buildRendering: function () {
            // summary:
            //		Creates and append the input hidden field to the dom to preserve the contentLink id
            // tags:
            //    protected

            var inputName = this.name ? (this.name.replace(/"/g, "&quot;")) : "";
            this.input = domConstruct.create("input", { type: "hidden", name: inputName });

            this.inherited(arguments);
            this.domNode.appendChild(this.input);
        },

        postMixInProperties: function () {
            // summary:
            //		Initialize properties
            // tags:
            //    protected

            var registry = dependency.resolve("epi.storeregistry");
            this._store = registry.get("epi.cms.content.light");

            var contentRepositoryDescriptors = this.contentRepositoryDescriptors || dependency.resolve("epi.cms.contentRepositoryDescriptors");
            var settings = contentRepositoryDescriptors.get(this.repositoryKey);

            if (settings && !this.roots) {
                this.roots = settings.roots;
            }

            if (settings && !this.searchArea) {
                this.searchArea = settings.searchArea;
            }

            this.inherited(arguments);
        },

        _setupContentSelectorDialogSettings: function () {
            this.inherited(arguments);

            this.title = localization.title;
            if (this.allowedTypes && this.allowedTypes.length === 1) {
                var name = TypeDescriptorManager.getResourceValue(this.allowedTypes[0], "name");
                if (name) {
                    this.title = stringUtil.substitute(localization.format, [name]);
                }
            }

            this.contentSelectorDialogSettings.title = this.title;
        },

        postCreate: function () {
            // summary:
            //      Overridden to initialize ui elements when no value is set.

            this.inherited(arguments);

            if (!this.value) {
                this._updateDisplayNode(null);
            }
        },

        _setValueAttr: function (value) {
            //summary:
            //    Value's setter.
            //
            // value: String
            //    Value to be set.
            //
            // tags:
            //    protected

            this._setValueAndFireOnChange(value);
        },

        _onDialogExecute: function (value) {
            //summary:
            //    Handle dialog close
            // tags:
            //    private

            this._setValueAndFireOnChange(value);
        },

        _onSearchResultSelected: function (value) {
            this._setValueAndFireOnChange(value);
        },

        _convertValueToContentReference: function (value) {
            //summary:
            //
            // tags:
            //    protected
            return value ? new ContentReference(value) : null;
        },

        _setValueAndFireOnChange: function (value) {
            //summary:
            //    Gets the content data by selected value, updates the view and calls onChange if value was changed
            // tags:
            //    private

            var contentLink = value === "-" ? this.contentLink : value;

            this.set("permanentLink", null);

            this._valueChangedPromise = when(this._getContentData(contentLink)).then(function (content) {

                if (this._destroyed) {
                    return;
                }

                // Clear the pointer to the promise since it is resolved.
                this._valueChangedPromise = null;

                var hasChange = this.value !== contentLink;

                this.value = contentLink;
                this.input.value = this.value;
                this._started && this.validate();
                this._updateDisplayNode(content);

                if (hasChange) {
                    this.onChange(contentLink);
                }

                if (content) {
                    this.set("permanentLink", content.permanentLink);
                }
            }.bind(this));
        },

        _getContentData: function (contentLink) {
            //summary:
            //    Loads the content data from the store
            // tags:
            //    private

            if (!contentLink) {
                return null;
            }
            return this._store.get(contentLink);
        },

        _onFocus: function () {
            // summary:
            //		This is where widgets do processing for when they start being active,
            //		such as changing CSS classes.  See onFocus() for more details.
            // tags:
            //		protected

            if (this.get("disabled")) {
                return;
            }
            this.inherited(arguments);
            this.validate();
        },

        _onBlur: function () {
            // summary:
            //		This is where widgets do processing for when they stop being active,
            //		such as changing CSS classes.  See onBlur() for more details.
            // tags:
            //		protected

            if (this.get("disabled")) {
                return;
            }
            this.inherited(arguments);
            this.validate();
        },

        _updateDisplayNode: function (content) {
            //summary:
            //    Update widget's display text
            // tags:
            //    protected

            this.inherited(arguments);

            // content selector also sets Link id
            if (content) {
                this.set("selectedContentLink", new ContentReference(content.contentLink).id);
            } else {
                this.set("selectedContentName", localization.helptext);
                this.set("selectedContentLink", "");
            }
        },

        reset: function () {
            this.set("value", null);
        }
    });
});
